@@ -29,11 +29,6 @@ struct p2p_beacon;
 struct mmpdu_header;
 struct wiphy;
 
-enum scan_band {
-	SCAN_BAND_2_4_GHZ =	0x1,
-	SCAN_BAND_5_GHZ =	0x2,
-};
-
 enum scan_state {
 	SCAN_STATE_NOT_RUNNING,
 	SCAN_STATE_PASSIVE,
@@ -90,6 +85,7 @@ struct scan_bss {
 	bool hs20_dgaf_disable : 1;
 	uint8_t cost_level : 3;
 	uint8_t cost_flags : 4;
+	bool dpp_configurator : 1;
 };
 
 struct scan_parameters {
@@ -112,7 +108,6 @@ typedef bool (*scan_notify_func_t)(int err, struct l_queue *bss_list,
 					const struct scan_freq_set *freqs,
 					void *userdata);
 typedef void (*scan_destroy_func_t)(void *userdata);
-typedef void (*scan_freq_set_func_t)(uint32_t freq, void *userdata);
 
 static inline int scan_bss_addr_cmp(const struct scan_bss *a1,
 					const struct scan_bss *a2)
@@ -172,23 +167,5 @@ struct scan_bss *scan_bss_new_from_probe_req(const struct mmpdu_header *mpdu,
 						size_t body_len,
 						uint32_t frequency, int rssi);
 
-uint8_t scan_freq_to_channel(uint32_t freq, enum scan_band *out_band);
-uint32_t scan_channel_to_freq(uint8_t channel, enum scan_band band);
-enum scan_band scan_oper_class_to_band(const uint8_t *country,
-					uint8_t oper_class);
-
-struct scan_freq_set *scan_freq_set_new(void);
-void scan_freq_set_free(struct scan_freq_set *freqs);
-bool scan_freq_set_add(struct scan_freq_set *freqs, uint32_t freq);
-bool scan_freq_set_contains(const struct scan_freq_set *freqs, uint32_t freq);
-uint32_t scan_freq_set_get_bands(struct scan_freq_set *freqs);
-void scan_freq_set_foreach(const struct scan_freq_set *freqs,
-				scan_freq_set_func_t func, void *user_data);
-void scan_freq_set_merge(struct scan_freq_set *to,
-					const struct scan_freq_set *from);
-void scan_freq_set_constrain(struct scan_freq_set *set,
-					const struct scan_freq_set *constraint);
-bool scan_freq_set_isempty(const struct scan_freq_set *set);
-
 bool scan_wdev_add(uint64_t wdev_id);
 bool scan_wdev_remove(uint64_t wdev_id);
